"use strict";
const isPromise = obj => (typeof obj !== 'undefined') && typeof obj.then === 'function';
const then = (action, onFulfilled, onRejected) => {
    try {
        const result = action();
        if (isPromise(result)) {
            return result.then(onFulfilled, onRejected);
        }
        else {
            return onFulfilled(result);
        }
    }
    catch (err) {
        if (typeof onRejected !== 'undefined') {
            return onRejected(err);
        }
        else {
            throw err;
        }
    }
};
exports.PromiseOrValue = {
    map: (promiseOrValue, onFulfilled) => {
        if (isPromise(promiseOrValue)) {
            return promiseOrValue.then(onFulfilled);
        }
        else {
            return onFulfilled(promiseOrValue);
        }
    },
    then: then,
    catch: (action, onRejected) => {
        return then(action, x => x, onRejected);
    }
};
const makeSpeech = (speech) => {
    if (typeof speech.SSML !== 'undefined') {
        return {
            type: 'SSML',
            ssml: speech.SSML
        };
    }
    else if (typeof speech.Text !== 'undefined') {
        return {
            type: 'PlainText',
            text: speech.Text
        };
    }
    else {
        return undefined;
    }
};
const makeCard = (card) => {
    if (card.Type === 'LinkAccount') {
        return {
            type: 'LinkAccount',
        };
    }
    else if ('Image' in card) {
        return {
            type: 'Standard',
            title: card.Title,
            text: card.Content,
            image: {
                largeImageUrl: card.Image.LargeUrl,
                smallImageUrl: card.Image.SmallUrl,
            }
        };
    }
    else {
        return {
            type: 'Simple',
            title: card.Title,
            content: card.Content,
        };
    }
};
const makeResponse = (response) => {
    const output = {
        shouldEndSession: response.EndSession || false
    };
    if ('Say' in response) {
        const speech = makeSpeech(response.Say);
        if (typeof speech !== 'undefined') {
            output.outputSpeech = speech;
        }
    }
    if ('Reprompt' in response) {
        const speech = makeSpeech(response.Reprompt);
        if (typeof speech !== 'undefined') {
            output.reprompt = { outputSpeech: speech };
        }
    }
    if ('Card' in response) {
        output.card = makeCard(response.Card);
    }
    return output;
};
exports.State = {
    attributeKey: '_alexaTsState',
    fromRequest: (request, initialState) => {
        if (request && request.session && request.session.attributes && exports.State.attributeKey in request.session.attributes) {
            return request.session.attributes[exports.State.attributeKey];
        }
        else {
            return initialState;
        }
    },
};
const sessionAttributesFromResponse = (response, previousState) => {
    let sessionAttributes = {};
    if ('NewState' in response) {
        sessionAttributes[exports.State.attributeKey] = response.NewState;
    }
    else if (typeof previousState !== 'undefined') {
        sessionAttributes[exports.State.attributeKey] = previousState;
    }
    return sessionAttributes;
};
exports.response = (response, previousState) => {
    return {
        version: '1.0',
        response: makeResponse(response),
        sessionAttributes: sessionAttributesFromResponse(response, previousState),
    };
};
const handlerObjToMap = (obj) => {
    const map = new Map();
    Object.keys(obj).forEach(key => map.set(`AMAZON.${key}Intent`, obj[key]));
    return map;
};
const slotsToMap = (slots) => {
    const map = new Map();
    if (slots instanceof Object) {
        Object.keys(slots).forEach(key => map.set(slots[key].name, slots[key].value));
    }
    return map;
};
const buildRequestIfNeeded = (outputOrResponse, previousState) => {
    if ('Say' in outputOrResponse) {
        return exports.response(outputOrResponse, previousState);
    }
    else {
        return outputOrResponse;
    }
};
const mapIntentResult = (result, state) => exports.PromiseOrValue.map(result, output => buildRequestIfNeeded(output, state));
const router = (routes) => {
    const standardIntents = handlerObjToMap(routes.Standard || {});
    const customIntents = new Map(routes.Custom || []);
    return (event, next) => {
        const state = exports.State.fromRequest(event, routes.InitialState);
        switch (event.request.type) {
            case 'LaunchRequest':
                if ('Launch' in routes) {
                    return mapIntentResult(routes.Launch(state, new Map(), event, next), state);
                }
                break;
            case 'SessionEndedRequest':
                if ('SessionEnded' in routes) {
                    return routes.SessionEnded();
                }
                return;
            case 'IntentRequest':
                const intentRequest = event.request;
                const slots = slotsToMap(intentRequest.intent.slots);
                if (standardIntents.has(intentRequest.intent.name)) {
                    const handler = standardIntents.get(intentRequest.intent.name);
                    return mapIntentResult(handler(state, slots, event, next), state);
                }
                if (customIntents.has(intentRequest.intent.name)) {
                    const handler = customIntents.get(intentRequest.intent.name);
                    return mapIntentResult(handler(state, slots, event, next), state);
                }
        }
        return next(event);
    };
};
exports.Pipe = {
    join: (steps) => (event, next) => {
        const processNext = (remainingHandlers) => (event) => {
            if (remainingHandlers.length === 0) {
                if (typeof next !== 'undefined') {
                    return next(event);
                }
                else {
                    throw new Error('Event unhandled');
                }
            }
            else {
                const nextHandler = remainingHandlers[0];
                const newRemaining = remainingHandlers.slice(1);
                return nextHandler(event, processNext(newRemaining));
            }
        };
        return processNext(steps)(event);
    },
    toHandler: (pipe) => event => pipe(event, () => { throw new Error('Event unhandled'); }),
    router: router,
    catch: (onError) => (request, next) => exports.PromiseOrValue.catch(() => next(request), onError),
    tracer: (logger) => {
        if (typeof logger === 'undefined') {
            logger = (message, obj) => console.log(message, JSON.stringify(obj));
        }
        return (event, next) => {
            logger('Request:', event);
            try {
                const result = next(event);
                if (isPromise(result)) {
                    return result.then((response) => {
                        logger('Response:', response);
                        return response;
                    }, (error) => {
                        logger('Error:', error.toString());
                        throw error;
                    });
                }
                else {
                    logger('Response:', result);
                    return result;
                }
            }
            catch (error) {
                logger('Error:', error.toString());
                throw error;
            }
        };
    },
    doNothing: () => (event, next) => next(event),
};
exports.Handler = {
    middleware: exports.Pipe.toHandler,
    router: (routes) => exports.Pipe.toHandler(router(routes)),
    pipe: (steps) => exports.Pipe.toHandler(exports.Pipe.join(steps)),
};
const lambdaFromHandler = (handler) => (event, context, callback) => {
    exports.PromiseOrValue.then(() => handler(event), data => callback(null, data), callback);
};
exports.Lambda = {
    handler: lambdaFromHandler,
    router: (routes) => lambdaFromHandler(exports.Handler.router(routes)),
    pipe: (steps) => lambdaFromHandler(exports.Handler.pipe(steps))
};
